//------------------------------------------------------------//
//                                                            //
//  4~4sNX                                            //
//                                                            //
//	Hiromu Itoh     2005                                      //
//                                                            //
//------------------------------------------------------------//
public class Matrix4x4d implements Cloneable
{
	public final static int DIMENSION = 4;

	private final static double EPSILON = 1.0E-10;		// Math.pow(2.0, -32.0) = 2.33E-10;

	private double mat[][];

	public Matrix4x4d()
	{
		mat = new double[DIMENSION][DIMENSION];

		makeIdentity();
	}

	public Matrix4x4d(double m[][])
	{
		mat = new double[DIMENSION][DIMENSION];

		setValue(m);
	}

	public void set(int i, int j, double value)
	{
		if ((i < 0 || i >= DIMENSION) && (j < 0 || j >= DIMENSION)) {
			return;
		}

		mat[i][j] = value;
	}

	public double get(int i, int j)
	{
		if ((i < 0 || i >= DIMENSION) && (j < 0 || j >= DIMENSION)) {
			return 0.0;
		}

		return mat[i][j];
	}

	public void setValue(double m[][])
	{
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				set(i, j, m[i][j]);
			}
		}
	}

	public void getValue(double m[][])
	{
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				m[i][j] = get(i, j);
			}
		}
	}

	public boolean equals(Object o)
	{
		if (o.getClass().getName() != this.getClass().getName()) {
			return false;
		}

		Matrix4x4d m = (Matrix4x4d)o;

		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				if (Math.abs(mat[i][j] - m.mat[i][j]) > EPSILON) {
					return false;
				}
			}
		}

		return true;
	}

	public Matrix4x4d copy()
	{
		return (new Matrix4x4d(mat));
	}

	public Object clone()
	{
		return (new Matrix4x4d(mat));
	}

	public Matrix4x4d add(double scalar)
	{
		Matrix4x4d m = new Matrix4x4d();
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				m.mat[i][j] = mat[i][j] + scalar;
			}
		}

		return m;
	}

	public Matrix4x4d subtract(double scalar)
	{
		Matrix4x4d m = new Matrix4x4d();
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				m.mat[i][j] = mat[i][j] - scalar;
			}
		}

		return m;
	}

	public Matrix4x4d multiply(double scalar)
	{
		Matrix4x4d m = new Matrix4x4d();
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				m.mat[i][j] = mat[i][j] * scalar;
			}
		}

		return m;
	}

	public Matrix4x4d divide(double scalar)
	{
		if (scalar == 0.0) {
			return null;
		}

		Matrix4x4d m = new Matrix4x4d();
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				m.mat[i][j] = mat[i][j] / scalar;
			}
		}

		return m;
	}

	public void addEqual(double scalar)
	{
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				mat[i][j] += scalar;
			}
		}
	}

	public void subtractEqual(double scalar)
	{
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				mat[i][j] -= scalar;
			}
		}
	}

	public void multiplyEqual(double scalar)
	{
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				mat[i][j] *= scalar;
			}
		}
	}

	public void divideEqual(double scalar)
	{
		if (scalar == 0.0) {
			return;
		}

		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				mat[i][j] /= scalar;
			}
		}
	}

	public Matrix4x4d add(Matrix4x4d m)
	{
		Matrix4x4d dstM = new Matrix4x4d();
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				dstM.mat[i][j] = mat[i][j] + m.mat[i][j];
			}
		}

		return dstM;
	}

	public Matrix4x4d subtract(Matrix4x4d m)
	{
		Matrix4x4d dstM = new Matrix4x4d();
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				dstM.mat[i][j] = mat[i][j] - m.mat[i][j];
			}
		}

		return dstM;
	}

	public void addEqual(Matrix4x4d m)
	{
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				mat[i][j] += m.mat[i][j];
			}
		}
	}

	public void subtractEqual(Matrix4x4d m)
	{
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				mat[i][j] -= m.mat[i][j];
			}
		}
	}

	public Matrix4x4d multiplyRight(Matrix4x4d m)
	{
		Matrix4x4d dstM = new Matrix4x4d();
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				dstM.mat[i][j] = 0.0;
				for (int k = 0; k < DIMENSION; k++) {
					dstM.mat[i][j] += mat[i][k] * m.mat[k][j];
				}
			}
		}

		return dstM;
	}

	public Matrix4x4d multiplyLeft(Matrix4x4d m)
	{
		Matrix4x4d dstM = new Matrix4x4d();
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				dstM.mat[i][j] = 0.0;
				for (int k = 0; k < DIMENSION; k++) {
					dstM.mat[i][j] += m.mat[i][k] * mat[k][j];
				}
			}
		}

		return dstM;
	}

	public void multiplyRightEqual(Matrix4x4d m)
	{
		double t[][] = new double[DIMENSION][DIMENSION];
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				t[i][j] = 0.0;
				for (int k = 0; k < DIMENSION; k++) {
					t[i][j] += mat[i][k] * m.mat[k][j];
				}
			}
		}

		setValue(t);
	}

	public void multiplyLeftEqual(Matrix4x4d m)
	{
		double t[][] = new double[DIMENSION][DIMENSION];
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				t[i][j] = 0.0;
				for (int k = 0; k < DIMENSION; k++) {
					t[i][j] += m.mat[i][k] * mat[k][j];
				}
			}
		}

		setValue(t);
	}

	public Vector3d multiplyVecRight(Vector3d v)
	{
		Vector3d dstV = new Vector3d();

		for (int i = 0; i < Vector3d.DIMENSION; i++) {
			dstV.set(i, 0.0);
			for (int j = 0; j < Vector3d.DIMENSION; j++) {
				dstV.set(i, dstV.get(i) + mat[i][j] * v.get(j));
			}
			dstV.set(i, dstV.get(i) + mat[i][DIMENSION - 1]);
		}

		return dstV;
	}

	public Vector3d multiplyVecLeft(Vector3d v)
	{
		Vector3d dstV = new Vector3d();

		for (int i = 0; i < Vector3d.DIMENSION; i++) {
			dstV.set(i, 0.0);
			for (int j = 0; j < Vector3d.DIMENSION; j++) {
				dstV.set(i, dstV.get(i) + mat[j][i] * v.get(j));
			}
			dstV.set(i, dstV.get(i) + mat[DIMENSION - 1][i]);
		}

		return dstV;
	}

	public void makeIdentity()
	{
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				mat[i][j] = (i == j ? 1.0 : 0.0);
			}
		}
	}

	public void makeZero()
	{
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				mat[i][j] = 0.0;
			}
		}
	}

	public void makeOrthonormal(Vector3d x, Vector3d y, Vector3d z)
	{
		makeIdentity();

		mat[0][0] = x.get(0);
		mat[1][0] = x.get(1);
		mat[2][0] = x.get(2);
		
		mat[0][1] = y.get(0);
		mat[1][1] = y.get(1);
		mat[2][1] = y.get(2);
		
		mat[0][2] = z.get(0);
		mat[1][2] = z.get(1);
		mat[2][2] = z.get(2);
	}

	public void makeScale(double scale)
	{
		makeIdentity();

		for (int i = 0; i < DIMENSION - 1; i++) {
			mat[i][i] = scale;
		}
	}

	public void makeScale(Vector3d scale)
	{
		makeIdentity();

		for (int i = 0; i < DIMENSION - 1; i++) {
			mat[i][i] = scale.get(i);
		}
	}

	public void makeTranslation(double x, double y, double z)
	{
		makeIdentity();

		mat[0][DIMENSION - 1] = x;
		mat[1][DIMENSION - 1] = y;
		mat[2][DIMENSION - 1] = z;
	}

	public void makeTranslation(Vector3d trans)
	{
		makeIdentity();

		for (int i = 0; i < DIMENSION - 1; i++) {
			mat[i][DIMENSION - 1] = trans.get(i);
		}
	}

	public void makeRotation(Vector3d axis, double angle)
	{
		makeIdentity();

		Vector3d u = axis.copy();
		u.normalize();

		Matrix4x4d I = new Matrix4x4d();
		I.makeIdentity();

		Matrix4x4d uuT = new Matrix4x4d();
		uuT.makeIdentity();
		for (int i = 0; i < DIMENSION - 1; i++) {
			for (int j = 0; j < DIMENSION - 1; j++) {
				uuT.mat[i][j] = u.get(i) * u.get(j);
			}
		}

		Matrix4x4d S = new Matrix4x4d();
		S.makeIdentity();
		S.mat[0][0] = 0.0f;
		S.mat[0][1] = -u.get(2);
		S.mat[0][2] = u.get(1);
		S.mat[1][0] = u.get(2);
		S.mat[1][1] = 0.0f;
		S.mat[1][2] = -u.get(0);
		S.mat[2][0] = -u.get(1);
		S.mat[2][1] = u.get(0);
		S.mat[2][2] = 0.0f;
		
		Matrix4x4d M2;
		M2 = I.subtract(uuT);
		M2.multiplyEqual(Math.cos(degreeToRadian(angle)));
		M2.mat[3][3] = 1.0f;

		Matrix4x4d M3;
		M3 = S.multiply(Math.sin(degreeToRadian(angle)));
		M3.mat[3][3] = 1.0f;

		Matrix4x4d M = new Matrix4x4d();
		M.makeZero();
		M.addEqual(uuT);
		M.addEqual(M2);
		M.addEqual(M3);
		M.mat[3][3] = 1.0f;

		M.getValue(mat);
	}

	public void makePerspective(double l, double r, double b, double t, double n, double f)
	{

		mat[0][0] = 2.0 * n / (r - l);
		mat[0][1] = 0.0;
		mat[0][2] = (r + l) / (r - l);
		mat[0][3] = 0.0;

		mat[1][0] = 0.0;
		mat[1][1] = 2.0 * n / (t - b);
		mat[1][2] = (t + b) / (t - b);
		mat[1][3] = 0.0;

		mat[2][0] = 0.0;
		mat[2][1] = 0.0;
		mat[2][2] = -(f + n) / (f - n);
		mat[2][3] = -2.0 * f * n / (f - n);

		mat[3][0] = 0.0;
		mat[3][1] = 0.0;
		mat[3][2] = -1.0;
		mat[3][3] = 0.0;
	}

	public void makeOrthographic(double l, double r, double b, double t, double n, double f)
	{

		mat[0][0] = 2.0 / (r - l);
		mat[0][1] = 0.0;
		mat[0][2] = 0.0;
		mat[0][3] = (r + l) / (r - l);

		mat[1][0] = 0.0;
		mat[1][1] = 2.0 / (t - b);
		mat[1][2] = 0.0;
		mat[1][3] = (t + b) / (t - b);

		mat[2][0] = 0.0;
		mat[2][1] = 0.0;
		mat[2][2] = - 2.0 / (f - n);
		mat[2][3] = (f + n) / (f - n);

		mat[3][0] = 0.0;
		mat[3][1] = 0.0;
		mat[3][2] = 0.0;
		mat[3][3] = 1.0;
	}

	public void makeView(Vector3d to, Vector3d from, Vector3d up)
	{
		Vector3d zeroVec = new Vector3d(0.0, 0.0, 0.0);
		Vector3d dir;
		Vector3d y = new Vector3d(0.0, 1.0, 0.0);
		Vector3d z = new Vector3d(0.0, 0.0, 1.0);
		Vector3d minusZ = new Vector3d(0.0, 0.0, -1.0);
		Vector3d axis;
		double angle;
		double cos;
		double sin;

		dir = to.subtract(from);
		dir.normalize();

		Vector3d dirZX = dir.copy();
		dirZX.setY(0.0);
		dirZX.normalize();

		Matrix4x4d rotDirMat1 = new Matrix4x4d();
		rotDirMat1.makeIdentity();

		axis = minusZ.cross(dirZX);
		if (axis.length() > 0.0) {
			cos = minusZ.dot(dirZX);
			sin = axis.length();
			angle = radianToDegree(Math.atan2( sin, cos ));
			axis.normalize();
			rotDirMat1.makeRotation(axis, angle);
		}

		Matrix4x4d rotDirMat2 = new Matrix4x4d();
		rotDirMat2.makeIdentity();
		axis = dirZX.cross(dir);
		if (axis.length() > 0.0) {
			cos = dirZX.dot(dir);
			sin = axis.length();
			angle = radianToDegree(Math.atan2(sin, cos));
			axis.normalize();
			rotDirMat2.makeRotation(axis, angle);
		}
		Matrix4x4d rotMat1 = rotDirMat2.multiplyRight(rotDirMat1);

		double proj;
		Vector3d projVec;
		Vector3d normalUp;
		Vector3d verticalUp;
		Vector3d verticalY;
		normalUp = up;
		normalUp.normalize();
		proj = normalUp.dot(dir);
		projVec = dir.multiply(proj);
		verticalUp = normalUp.subtract(projVec);
		proj = y.dot(dir);
		projVec = dir.multiply(proj);
		verticalY = y.subtract(projVec);
		axis = verticalY.cross(verticalUp);

		Matrix4x4d rotMat2 = new Matrix4x4d();
		rotMat2.makeIdentity();
		if (axis.length() > 0.0) {
			cos = verticalY.dot(verticalUp);
			sin = axis.length();
			angle = radianToDegree(Math.atan2(sin, cos));
			axis.normalize();
			rotMat2.makeRotation(axis, angle);
		}

		Matrix4x4d rotMat = rotMat2.multiplyRight(rotMat1);

		Vector3d trans;
		Matrix4x4d transMat = new Matrix4x4d();
		transMat.makeIdentity();
		trans = from;
		transMat.makeTranslation(trans);

		Matrix4x4d viewMat;
		viewMat = transMat.multiplyRight(rotMat);
		viewMat.invert();

		viewMat.getValue(mat);
	}

	public double det()
	{
		return det(mat, DIMENSION);
	}

	private double det(double mat[][], int dimension)
	{
  		double det;

		if (dimension == 1) {
			det = mat[0][0];
		}
		else {
			det = 0.0f;
			for (int j = 0; j < dimension; j++) {
				det += mat[0][j] * cofactor(mat, dimension, 0, j);
			}
		}

		return det;
	}

	public double cofactor(int i, int j)
	{
		return cofactor(mat, DIMENSION, i, j);
	}

	private double cofactor(double mat[][], int dimension, int i, int j)
	{
		double cofactor = 0.0;
		double m[][] = new double[DIMENSION][DIMENSION];

		for (int k = 0; k < dimension; k++) {
			for (int l = 0; l < dimension; l++) {
				if (k < i && l < j) {
					m[k][l] = mat[k][l];
				}
				else if (k < i && l > j) {
					m[k][l - 1] = mat[k][l];
				}
				else if (k > i && l < j) {
					m[k - 1][l] = mat[k][l];
				}
				else if (k > i && l > j) {
					m[k - 1][l - 1] = mat[k][l];
				}
			}
		}

		cofactor = Math.pow(-1.0, i + j) * det(m, dimension - 1);

		return cofactor;
	}

	public boolean invert()
	{
		double cofactorMat[][] = new double[DIMENSION][DIMENSION];	
		double det;
		boolean status;

		for (int row = 0; row < DIMENSION; row++) {
			for (int col = 0; col < DIMENSION; col++) {
				cofactorMat[row][col] = cofactor(col, row);
			}
		}

  		det = det();

		if (det != 0.0) {
			for (int row = 0; row < DIMENSION; row++) {
				for (int col = 0; col < DIMENSION; col++) {
					mat[row][col] = cofactorMat[row][col] / det;
				}
			}
		    status = true;
		}
		else {
			status = false;
		}

		return status;
	}

	public boolean inverse(Matrix4x4d m)
	{
		double cofactorMat[][] = new double[DIMENSION][DIMENSION];	
		double det;
		boolean status;

		for (int row = 0; row < DIMENSION; row++) {
			for (int col = 0; col < DIMENSION; col++) {
				cofactorMat[row][col] = cofactor(col, row);
			}
		}

  		det = det();

		if (det != 0.0) {
			for (int row = 0; row < DIMENSION; row++) {
				for (int col = 0; col < DIMENSION; col++) {
					m.mat[row][col] = cofactorMat[row][col] / det;
				}
			}
		    status = true;
		}
		else {
			status = false;
		}

		return status;
	}

	public void transpose()
	{
		double m[][] = new double[DIMENSION][DIMENSION];

		for (int row = 0; row < DIMENSION; row++) {
			for (int col = 0; col < DIMENSION; col++) {
				m[row][col] = mat[col][row];
			}
		}

		setValue(m);
	}

	public void transposed(Matrix4x4d m)
	{
		for (int row = 0; row < DIMENSION; row++) {
			for (int col = 0; col < DIMENSION; col++) {
				m.mat[row][col] = mat[col][row];
			}
		}
	}

	public Matrix4x4d transposed()
	{
		Matrix4x4d m = new Matrix4x4d();

		for (int row = 0; row < DIMENSION; row++) {
			for (int col = 0; col < DIMENSION; col++) {
				m.mat[row][col] = mat[col][row];
			}
		}

		return m;
	}

	public void print()
	{
		System.out.println("----------------------------------------");

		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				if (j > 0) {
					System.out.print(", ");
				}
				System.out.print(mat[i][j]);
			}
			System.out.println();
		}

		System.out.println("----------------------------------------");
	}

	public void print(double round)
	{
		System.out.println("----------------------------------------");

		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				if (j > 0) {
					System.out.print(", ");
				}
				double value;
				if (round == 0.0) {
					value = mat[i][j];
				}
				else {
					value = mat[i][j] / round;
					value = Math.floor(value + 0.5) * round;
				}
				System.out.print(value);
			}
			System.out.println();
		}

		System.out.println("----------------------------------------");
	}

	public String toString()
	{
		String line = "----------------------------------------\n";
		String value = new String();
		for (int i = 0; i < DIMENSION; i++) {
			for (int j = 0; j < DIMENSION; j++) {
				if (j > 0) {
					value += ", ";
				}
				value += mat[i][j];
			}
			value += "\n";
		}

		return (line + value + line);
	}

	// PROTECTED METHODS
	protected double degreeToRadian(double degree)
	{
		return (degree * Math.PI / 180.0);
	}

	protected double radianToDegree(double radian)
	{
		return (radian * 180.0 / Math.PI);
	}

	protected void printErrorMessage(String methodName)
	{
		String errorMessage = "ERROR : ";
		errorMessage += ("IN CLASS [" + getClass().getName() + ", ");
		errorMessage += ("IN METHOD [" + methodName + "]");

		System.err.println(errorMessage);
	}
}
