//------------------------------------------------------------//
//                                                            //
//  ̃NX                                              //
//                                                            //
//	Hiromu Itoh     2006                                      //
//                                                            //
//------------------------------------------------------------//
import java.awt.*;

public class Cube
{
	private final static int NUM_FACE = 6;
	private final static int FRONT_INDEX = 0;
	private final static int RIGHT_INDEX = 1;
	private final static int BACK_INDEX = 2;
	private final static int LEFT_INDEX = 3;
	private final static int TOP_INDEX = 4;
	private final static int BOTTOM_INDEX = 5;

	private Face face[];

	// O
	private double front[][] =
	{
		{0.5, -0.5, 0.5},
		{0.5, 0.5, 0.5},
		{-0.5, 0.5, 0.5},
		{-0.5, -0.5, 0.5}
	};

	// w
	private double back[][] =
	{
		{-0.5, -0.5, -0.5},
		{-0.5, 0.5, -0.5},
		{0.5, 0.5, -0.5},
		{0.5, -0.5, -0.5}
	};

	// E
	private double right[][] =
	{
		{0.5, -0.5, -0.5},
		{0.5, 0.5, -0.5},
		{0.5, 0.5, 0.5},
		{0.5, -0.5, 0.5}
	};

	// 
	private double left[][] =
	{
		{-0.5, -0.5, 0.5},
		{-0.5, 0.5, 0.5},
		{-0.5, 0.5, -0.5},
		{-0.5, -0.5, -0.5}
	};

	// 
	private double top[][] =
	{
		{0.5, 0.5, 0.5},
		{0.5, 0.5, -0.5},
		{-0.5, 0.5, -0.5},
		{-0.5, 0.5, 0.5}
	};

	// 
	private double bottom[][] =
	{
		{0.5, -0.5, -0.5},
		{0.5, -0.5, 0.5},
		{-0.5, -0.5, 0.5},
		{-0.5, -0.5, -0.5}
	};

	public Cube()
	{
		this(1.0);
	}

	public Cube(double size)
	{
		face = new Face[NUM_FACE];

		// O
		Vector3d frontV[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			frontV[i] = new Vector3d(front[i]);
			frontV[i].multiplyEqual(size);
		}
		face[FRONT_INDEX] = new Face(frontV);

		// 
		Vector3d backV[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			backV[i] = new Vector3d(back[i]);
			backV[i].multiplyEqual(size);
		}
		face[BACK_INDEX] = new Face(backV);

		// E
		Vector3d rightV[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			rightV[i] = new Vector3d(right[i]);
			rightV[i].multiplyEqual(size);
		}
		face[RIGHT_INDEX] = new Face(rightV);

		// 
		Vector3d leftV[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			leftV[i] = new Vector3d(left[i]);
			leftV[i].multiplyEqual(size);
		}
		face[LEFT_INDEX] = new Face(leftV);

		// 
		Vector3d topV[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			topV[i] = new Vector3d(top[i]);
			topV[i].multiplyEqual(size);
		}
		face[TOP_INDEX] = new Face(topV);

		// 
		Vector3d bottomV[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			bottomV[i] = new Vector3d(bottom[i]);
			bottomV[i].multiplyEqual(size);
		}
		face[BOTTOM_INDEX] = new Face(bottomV);
	}

	public void setDiffuse(float color[])
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setDiffuse(color);
		}
	}

	public void setDiffuse(float red, float green, float blue)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setDiffuse(red, green, blue);
		}
	}

	public void setAmbient(float color[])
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setAmbient(color);
		}
	}

	public void setAmbient(float red, float green, float blue)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setAmbient(red, green, blue);
		}
	}

	public void setLineColor(float color[])
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setLineColor(color);
		}
	}

	public void setLineColor(float red, float green, float blue)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setLineColor(red, green, blue);
		}
	}

	public void persDraw(Vector2d winCenter, Vector2d winScale, Matrix4x4d persM, Matrix4x4d modelM, Matrix4x4d rotM, Vector3d viewPos, Vector3d lightDir, Graphics g)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].persDraw(winCenter, winScale, persM, modelM, rotM, viewPos, lightDir, g);
		}
	}

	public void persDrawLine(Vector2d winCenter, Vector2d winScale, Matrix4x4d persM, Matrix4x4d modelM, Vector3d viewPos, Graphics g)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].persDrawLine(winCenter, winScale, persM, modelM, viewPos, g);
		}
	}
}
