//------------------------------------------------------------//
//                                                            //
//  lʑ̃NX                                            //
//                                                            //
//	Hiromu Itoh     2006                                      //
//                                                            //
//------------------------------------------------------------//
import java.awt.*;

public class Tetrahedron
{
	private final static int NUM_FACE = 4;
	private final static int SIDE1_INDEX = 0;
	private final static int SIDE2_INDEX = 1;
	private final static int SIDE3_INDEX = 2;
	private final static int BOTTOM_INDEX = 3;

	private Face face[];

	// 
	private double side1[][] =
	{
		{1.0, -Math.sqrt(6.0)/6.0, -Math.sqrt(3.0)/3.0},
		{-1.0, -Math.sqrt(6.0)/6.0, -Math.sqrt(3.0)/3.0},
		{0.0, Math.sqrt(6.0)/2.0, 0.0}
	};
	private double side2[][] =
	{
		{0.0, -Math.sqrt(6.0)/6.0, 2.0*Math.sqrt(3.0)/3.0},
		{1.0, -Math.sqrt(6.0)/6.0, -Math.sqrt(3.0)/3.0},
		{0.0, Math.sqrt(6.0)/2.0, 0.0}
	};
	private double side3[][] =
	{
		{-1.0, -Math.sqrt(6.0)/6.0, -Math.sqrt(3.0)/3.0},
		{0.0, -Math.sqrt(6.0)/6.0, 2.0*Math.sqrt(3.0)/3.0},
		{0.0, Math.sqrt(6.0)/2.0, 0.0}
	};
	// 
	private double bottom[][] =
	{
		{-1.0, -Math.sqrt(6.0)/6.0, -Math.sqrt(3.0)/3.0},
		{1.0, -Math.sqrt(6.0)/6.0, -Math.sqrt(3.0)/3.0},
		{0.0, -Math.sqrt(6.0)/6.0, 2.0*Math.sqrt(3.0)/3.0}
	};

	public Tetrahedron()
	{
		this(1.0);
	}

	public Tetrahedron(double size)
	{
		face = new Face[NUM_FACE];

		// 
		Vector3d side1V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			side1V[i] = new Vector3d(side1[i]);
			side1V[i].multiplyEqual(size);
		}
		face[SIDE1_INDEX] = new Face(side1V);

		Vector3d side2V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			side2V[i] = new Vector3d(side2[i]);
			side2V[i].multiplyEqual(size);
		}
		face[SIDE2_INDEX] = new Face(side2V);

		Vector3d side3V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			side3V[i] = new Vector3d(side3[i]);
			side3V[i].multiplyEqual(size);
		}
		face[SIDE3_INDEX] = new Face(side3V);

		// 
		Vector3d bottomV[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			bottomV[i] = new Vector3d(bottom[i]);
			bottomV[i].multiplyEqual(size);
		}
		face[BOTTOM_INDEX] = new Face(bottomV);
	}

	public void setDiffuse(float color[])
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setDiffuse(color);
		}
	}

	public void setDiffuse(float red, float green, float blue)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setDiffuse(red, green, blue);
		}
	}

	public void setAmbient(float color[])
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setAmbient(color);
		}
	}

	public void setAmbient(float red, float green, float blue)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setAmbient(red, green, blue);
		}
	}

	public void setLineColor(float color[])
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setLineColor(color);
		}
	}

	public void setLineColor(float red, float green, float blue)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setLineColor(red, green, blue);
		}
	}

	public void persDraw(Vector2d winCenter, Vector2d winScale, Matrix4x4d persM, Matrix4x4d modelM, Matrix4x4d rotM, Vector3d viewPos, Vector3d lightDir, Graphics g)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].persDraw(winCenter, winScale, persM, modelM, rotM, viewPos, lightDir, g);
		}
	}

	public void persDrawLine(Vector2d winCenter, Vector2d winScale, Matrix4x4d persM, Matrix4x4d modelM, Vector3d viewPos, Graphics g)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].persDrawLine(winCenter, winScale, persM, modelM, viewPos, g);
		}
	}
}


