//------------------------------------------------------------//
//                                                            //
//  ̃NX                                              //
//                                                            //
//	Hiromu Itoh     2006                                      //
//                                                            //
//------------------------------------------------------------//
import java.awt.*;

public class Cube
{
	private final static int NUM_FACE = 12;
	private final static int FRONT1_INDEX = 0;
	private final static int FRONT2_INDEX = 1;
	private final static int RIGHT1_INDEX = 2;
	private final static int RIGHT2_INDEX = 3;
	private final static int BACK1_INDEX = 4;
	private final static int BACK2_INDEX = 5;
	private final static int LEFT1_INDEX = 6;
	private final static int LEFT2_INDEX = 7;
	private final static int TOP1_INDEX = 8;
	private final static int TOP2_INDEX = 9;
	private final static int BOTTOM1_INDEX = 10;
	private final static int BOTTOM2_INDEX = 11;

	private Face face[];

	// O
	private double front1[][] =
	{
		{0.5, -0.5, 0.5},
		{0.5, 0.5, 0.5},
		{-0.5, 0.5, 0.5}
	};
	private double front2[][] =
	{
		{0.5, -0.5, 0.5},
		{-0.5, 0.5, 0.5},
		{-0.5, -0.5, 0.5}
	};

	// 
	private double back1[][] =
	{
		{-0.5, -0.5, -0.5},
		{-0.5, 0.5, -0.5},
		{0.5, 0.5, -0.5}
	};
	private double back2[][] =
	{
		{-0.5, -0.5, -0.5},
		{0.5, 0.5, -0.5},
		{0.5, -0.5, -0.5}
	};

	// E
	private double right1[][] =
	{
		{0.5, -0.5, -0.5},
		{0.5, 0.5, -0.5},
		{0.5, 0.5, 0.5}
	};
	private double right2[][] =
	{
		{0.5, -0.5, -0.5},
		{0.5, 0.5, 0.5},
		{0.5, -0.5, 0.5}
	};

	// 
	private double left1[][] =
	{
		{-0.5, -0.5, 0.5},
		{-0.5, 0.5, 0.5},
		{-0.5, 0.5, -0.5}
	};
	private double left2[][] =
	{
		{-0.5, -0.5, 0.5},
		{-0.5, 0.5, -0.5},
		{-0.5, -0.5, -0.5}
	};

	// 
	private double top1[][] =
	{
		{0.5, 0.5, 0.5},
		{0.5, 0.5, -0.5},
		{-0.5, 0.5, -0.5}
	};
	private double top2[][] =
	{
		{0.5, 0.5, 0.5},
		{-0.5, 0.5, -0.5},
		{-0.5, 0.5, 0.5}
	};

	// 
	private double bottom1[][] =
	{
		{0.5, -0.5, -0.5},
		{0.5, -0.5, 0.5},
		{-0.5, -0.5, 0.5}
	};
	private double bottom2[][] =
	{
		{0.5, -0.5, -0.5},
		{-0.5, -0.5, 0.5},
		{-0.5, -0.5, -0.5}
	};

	public Cube()
	{
		this(1.0);
	}

	public Cube(double size)
	{
		face = new Face[NUM_FACE];

		// O
		Vector3d front1V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			front1V[i] = new Vector3d(front1[i]);
			front1V[i].multiplyEqual(size);
		}
		face[FRONT1_INDEX] = new Face(front1V);
		Vector3d front2V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			front2V[i] = new Vector3d(front2[i]);
			front2V[i].multiplyEqual(size);
		}
		face[FRONT2_INDEX] = new Face(front2V);

		// 
		Vector3d back1V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			back1V[i] = new Vector3d(back1[i]);
			back1V[i].multiplyEqual(size);
		}
		face[BACK1_INDEX] = new Face(back1V);
		Vector3d back2V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			back2V[i] = new Vector3d(back2[i]);
			back2V[i].multiplyEqual(size);
		}
		face[BACK2_INDEX] = new Face(back2V);

		// E
		Vector3d right1V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			right1V[i] = new Vector3d(right1[i]);
			right1V[i].multiplyEqual(size);
		}
		face[RIGHT1_INDEX] = new Face(right1V);
		Vector3d right2V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			right2V[i] = new Vector3d(right2[i]);
			right2V[i].multiplyEqual(size);
		}
		face[RIGHT2_INDEX] = new Face(right2V);

		// 
		Vector3d left1V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			left1V[i] = new Vector3d(left1[i]);
			left1V[i].multiplyEqual(size);
		}
		face[LEFT1_INDEX] = new Face(left1V);
		Vector3d left2V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			left2V[i] = new Vector3d(left2[i]);
			left2V[i].multiplyEqual(size);
		}
		face[LEFT2_INDEX] = new Face(left2V);

		// 
		Vector3d top1V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			top1V[i] = new Vector3d(top1[i]);
			top1V[i].multiplyEqual(size);
		}
		face[TOP1_INDEX] = new Face(top1V);
		Vector3d top2V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			top2V[i] = new Vector3d(top2[i]);
			top2V[i].multiplyEqual(size);
		}
		face[TOP2_INDEX] = new Face(top2V);

		// 
		Vector3d bottom1V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			bottom1V[i] = new Vector3d(bottom1[i]);
			bottom1V[i].multiplyEqual(size);
		}
		face[BOTTOM1_INDEX] = new Face(bottom1V);
		Vector3d bottom2V[] = new Vector3d[Face.NUM_VERTEX];
		for (int i = 0; i < Face.NUM_VERTEX; i++) {
			bottom2V[i] = new Vector3d(bottom2[i]);
			bottom2V[i].multiplyEqual(size);
		}
		face[BOTTOM2_INDEX] = new Face(bottom2V);
	}

	public void setDiffuse(float color[])
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setDiffuse(color);
		}
	}

	public void setDiffuse(float red, float green, float blue)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setDiffuse(red, green, blue);
		}
	}

	public void setAmbient(float color[])
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setAmbient(color);
		}
	}

	public void setAmbient(float red, float green, float blue)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setAmbient(red, green, blue);
		}
	}

	public void setLineColor(float color[])
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setLineColor(color);
		}
	}

	public void setLineColor(float red, float green, float blue)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].setLineColor(red, green, blue);
		}
	}

	public void persDraw(Vector2d winCenter, Vector2d winScale, Matrix4x4d persM, Matrix4x4d modelM, Matrix4x4d rotM, Vector3d viewPos, Vector3d lightDir, Graphics g)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].persDraw(winCenter, winScale, persM, modelM, rotM, viewPos, lightDir, g);
		}
	}

	public void persDrawLine(Vector2d winCenter, Vector2d winScale, Matrix4x4d persM, Matrix4x4d modelM, Vector3d viewPos, Graphics g)
	{
		for (int i = 0; i < face.length; i++) {
			face[i].persDrawLine(winCenter, winScale, persM, modelM, viewPos, g);
		}
	}
}
