//------------------------------------------------------------//
//                                                            //
//  ʃNX                                                  //
//                                                            //
//	Hiromu Itoh     2006                                      //
//                                                            //
//------------------------------------------------------------//
import java.awt.*;

public class Face
{
	public final static int NUM_VERTEX = 3;

	private final static double DEF_VERTEX[][] =
	{
		{-0.5, -0.5, 0.0},
		{0.5, -0.5, 0.0},
		{0.0, 0.5, 0.0}
	};

	private float diffuse[] = {1.0F, 1.0F, 1.0F};
	private float ambient[] = {0.125F, 0.125F, 0.125F};

	private float lineColor[] = {0.5F, 0.5F, 0.5F};
	private float pointColor[] = {1.0F, 1.0F, 1.0F};

	private Vector3d vertex[];
	private Vector3d normal;
	private Vector3d center;

	public Face()
	{
		this(1.0);
	}

	public Face(double scale)
	{
		vertex = new Vector3d[NUM_VERTEX];
		for (int i = 0; i < vertex.length; i++) {
			vertex[i] = new Vector3d(DEF_VERTEX[i]);
			vertex[i].multiplyEqual(scale);
		}

		normal = new Vector3d(0.0, 0.0, 1.0);
		center = new Vector3d(0.0, 0.0, 0.0);
	}
	
	public Face(double v[/*3*/][/*3*/])
	{
		vertex = new Vector3d[NUM_VERTEX];
		for (int i = 0; i < vertex.length; i++) {
			vertex[i] = new Vector3d(v[i]);
		}

		calculateNormal();

		center = new Vector3d();
		calculateCenter();
	}

	public Face(Vector3d v[/*3*/])
	{
		vertex = new Vector3d[NUM_VERTEX];
		for (int i = 0; i < vertex.length; i++) {
			vertex[i] = v[i].copy();
		}

		calculateNormal();

		center = new Vector3d();
		calculateCenter();
	}

	private void calculateNormal()
	{
		Vector3d v0 = new Vector3d(vertex[0]);
		Vector3d v1 = new Vector3d(vertex[1]);
		Vector3d v2 = new Vector3d(vertex[2]);
		Vector3d v21 = v2.subtract(v1);	// v21 = v2 - v1
		Vector3d v01 = v0.subtract(v1);	// v01 = v0 - v1

		normal = v21.cross(v01);		// n = v21 ~ v01
		normal.normalize();
	}

	private void calculateCenter()
	{
		center.setValue(0.0, 0.0, 0.0);
		for (int i = 0; i < vertex.length; i++) {
			center.addEqual(vertex[i]);
		}
		center.divideEqual(vertex.length);
	}

	public void setDiffuse(float color[])
	{
		for (int i = 0; i < 3; i++) {
			diffuse[i] = color[i];
		}
	}

	public void setDiffuse(float red, float green, float blue)
	{
		diffuse[0] = red;
		diffuse[1] = green;
		diffuse[2] = blue;
	}

	public void setAmbient(float color[])
	{
		for (int i = 0; i < 3; i++) {
			ambient[i] = color[i];
		}
	}

	public void setAmbient(float red, float green, float blue)
	{
		ambient[0] = red;
		ambient[1] = green;
		ambient[2] = blue;
	}

	public void setLineColor(float color[])
	{
		for (int i = 0; i < 3; i++) {
			lineColor[i] = color[i];
		}
	}

	public void setLineColor(float red, float green, float blue)
	{
		lineColor[0] = red;
		lineColor[1] = green;
		lineColor[2] = blue;
	}

	public void setPointColor(float color[])
	{
		for (int i = 0; i < 3; i++) {
			pointColor[i] = color[i];
		}
	}

	public void setPointColor(float red, float green, float blue)
	{
		pointColor[0] = red;
		pointColor[1] = green;
		pointColor[2] = blue;
	}

	// 󂯎fϊsAeϊs𗘗pĖʂ`悷iʌǉj
	public void persDraw(Vector2d winCenter, Vector2d winScale, 
	                     Matrix4x4d persM, Matrix4x4d modelM, Matrix4x4d rotM, 
	                     Vector3d eyePos, Vector3d lightDir, Graphics g)
	{
		Vector3d Mv[] = new Vector3d[vertex.length];
		Vector3d PMv[] = new Vector3d[vertex.length];
		for (int i = 0; i < vertex.length; i++) {
			// M:fϊsAv:_AMv:fϊ㒸_
			Mv[i] = modelM.multiplyVecRight(vertex[i]);
			// P:eϊsAPMv:eϊ㒸_
			PMv[i] = persM.multiplyVecRight(Mv[i]);
			PMv[i].divideEqual(-Mv[i].getZ());	// PŴXAYZŊ
		}

		// M:fϊsAc:ʂ̒S_AMc:fϊ̖ʂ̒S_
		Vector3d Mc = modelM.multiplyVecRight(center);
		// n:]ϊ@xNgin' = Rnj
		Vector3d n = rotM.multiplyVecRight(normal);
		n.normalize();
		// e:xNgie' = Mc - ej
		Vector3d e = Mc.subtract(eyePos);
		e.normalize();

		// @xNgƎxNg̐p90ȓ̂Ƃ̓_OȂ
		//	
		//	       n   e
		//	       |/
		//         | /
		//    \   |/ 
		//	--------------
		//      /
		//       /
		//     eye
		//     _
		if (n.dot(e) > 0.0) {
			return;
		}

		// l:xNgAcos(nl)=nEl:@xNgƌxNg̗]
		Vector3d l = lightDir.copy();
		l.normalize();
		float cosnl = (float)n.dot(l);
		cosnl = (cosnl > 0.0F ? 0.0F : -cosnl);

		// lAeʂ̕Ɍ
		l.nigate();
		e.nigate();
		// xNgl̖@xNgnւ̐ˉe߂
		Vector3d p = n.multiply(n.dot(l));
		// r:˕̃xNg
		Vector3d r = p.multiply(2.0).subtract(l);
		// ʌ߂(m = 20)
		double specular = 0.6 * Math.pow(n.dot(l) > 0.0 ? r.dot(e) : 0.0, 20);

		// color = ambient + diffuse * cos(nl) + specular
		float red = ambient[0] + diffuse[0] * cosnl + (float)specular;
		red = (red > 1.0F ? 1.0F : red);
		float green = ambient[1] + diffuse[1] * cosnl + (float)specular;
		green = (green > 1.0F ? 1.0F : green);
		float blue = ambient[2] + diffuse[2] * cosnl + (float)specular;
		blue = (blue > 1.0F ? 1.0F : blue);

		// v2d:e̒_i2Wnj
		Vector2d v2d[] = new Vector2d[vertex.length];
		for (int i = 0; i < vertex.length; i++) {
			v2d[i] = new Vector2d(PMv[i].getX(), PMv[i].getY());
		}

		int x[] = new int[vertex.length];
		int y[] = new int[vertex.length];
		// e̒_̃EBhEWւ̃}bsO
		for (int i = 0; i < vertex.length; i++) {
			x[i] = (int)(winCenter.getX() + v2d[i].getX() * winScale.getX());
			y[i] = (int)(winCenter.getY() - v2d[i].getY() * winScale.getY());
		}

		// |S̕`
		g.setColor(new Color(red, green, blue));
		g.fillPolygon(x, y, vertex.length);
	}

	// 󂯎fϊsAeϊs𗘗pĐ`悷
	public void persDrawLine(Vector2d winCenter, Vector2d winScale, Matrix4x4d persM, Matrix4x4d modelM, Vector3d viewPos, Graphics g)
	{
		Vector3d Mv[] = new Vector3d[vertex.length];
		Vector3d PMv[] = new Vector3d[vertex.length];
		for (int i = 0; i < vertex.length; i++) {
			// M:fϊsAv:_AMv:fϊ㒸_
			Mv[i] = modelM.multiplyVecRight(vertex[i]);
			// P:eϊsAPMv:eϊ㒸_
			PMv[i] = persM.multiplyVecRight(Mv[i]);
			PMv[i].divideEqual(-Mv[i].getZ());
		}

		// v2d:e̒_i2Wnj
		Vector2d v2d[] = new Vector2d[vertex.length];
		for (int i = 0; i < vertex.length; i++) {
			v2d[i] = new Vector2d(PMv[i].getX(), PMv[i].getY());
		}

		int x[] = new int[vertex.length];
		int y[] = new int[vertex.length];
		// e̒_̃EBhEWւ̃}bsO
		for (int i = 0; i < vertex.length; i++) {
			x[i] = (int)(winCenter.getX() + v2d[i].getX() * winScale.getX());
			y[i] = (int)(winCenter.getY() - v2d[i].getY() * winScale.getY());
		}

		// ̕`
		g.setColor(new Color(lineColor[0], lineColor[1], lineColor[2]));
		g.drawPolygon(x, y, vertex.length);
	}

	// 󂯎fϊsAeϊs𗘗pē_`悷
	public void persDrawPoint(Vector2d winCenter, Vector2d winScale, Matrix4x4d persM, Matrix4x4d modelM, Vector3d viewPos, Graphics g)
	{
		Vector3d Mv[] = new Vector3d[vertex.length];
		Vector3d PMv[] = new Vector3d[vertex.length];
		for (int i = 0; i < vertex.length; i++) {
			// M:fϊsAv:_AMv:fϊ㒸_
			Mv[i] = modelM.multiplyVecRight(vertex[i]);
			// P:eϊsAPMv:eϊ㒸_
			PMv[i] = persM.multiplyVecRight(Mv[i]);
			PMv[i].divideEqual(-Mv[i].getZ());
		}

		// v2d:e̒_i2Wnj
		Vector2d v2d[] = new Vector2d[vertex.length];
		for (int i = 0; i < vertex.length; i++) {
			v2d[i] = new Vector2d(PMv[i].getX(), PMv[i].getY());
		}

		int x[] = new int[vertex.length];
		int y[] = new int[vertex.length];
		// e̒_̃EBhEWւ̃}bsO
		for (int i = 0; i < vertex.length; i++) {
			x[i] = (int)(winCenter.getX() + v2d[i].getX() * winScale.getX());
			y[i] = (int)(winCenter.getY() - v2d[i].getY() * winScale.getY());
		}

		// ̕`
		g.setColor(new Color(pointColor[0], pointColor[1], pointColor[2]));
		for (int i = 0; i < vertex.length; i++) {
			g.fillOval(x[i], y[i], 3, 3);
		}
	}
}
