//------------------------------------------------------------//
//                                                            //
//  3xNgNX                                       //
//                                                            //
//	Hiromu Itoh     2005                                      //
//                                                            //
//------------------------------------------------------------//
public class Vector3d implements Cloneable
{
	// PUBLIC MEMBERS
	public final static int DIMENSION = 3;

	// PRIVATE MEMBERS
	private double vec[];

	// CONSTRUCTORS
	public Vector3d()
	{
		this(0.0, 0.0, 0.0);
	}

	public Vector3d(double x, double y, double z)
	{
		vec = new double[DIMENSION];

		vec[0] = x;
		vec[1] = y;
		vec[2] = z;
	}

	public Vector3d(double v[])
	{
		vec = new double[DIMENSION];

		vec[0] = v[0];
		vec[1] = v[1];
		vec[2] = v[2];
	}

	public Vector3d(Vector3d v)
	{
		this(v.vec);
	}

	// PUBLIC METHODS
	public void set(int index, double value)
	{
		if (index < 0 || index >= DIMENSION) {
			return;
		}

		vec[index] = value;
	}

	public double get(int index)
	{
		if (index < 0 || index >= DIMENSION) {
			return 0.0;
		}

		return vec[index];
	}

	public void setX(double value)
	{
		set(0, value);
	}

	public void setY(double value)
	{
		set(1, value);
	}

	public void setZ(double value)
	{
		set(2, value);
	}

	public double getX()
	{
		return get(0);
	}

	public double getY()
	{
		return get(1);
	}

	public double getZ()
	{
		return get(2);
	}

	public void setValue(double x, double y, double z)
	{
		setX(x);
		setY(y);
		setZ(z);
	}

	public void setValue(double v[])
	{
		for (int i = 0; i < DIMENSION; i++) {
			set(i, v[i]);
		}
	}

	public void getValue(double v[])
	{
		for (int i = 0; i < DIMENSION; i++) {
			v[i] = get(i);
		}
	}

	public Vector3d minus()
	{
		Vector3d v = new Vector3d();
		for (int i = 0; i < DIMENSION; i++) {
			v.vec[i] = - vec[i];
		}

		return v;
	}

	public Vector3d add(double scalar)
	{
		Vector3d v = new Vector3d();
		for (int i = 0; i < DIMENSION; i++) {
			v.vec[i] = vec[i] + scalar;
		}

		return v;
	}

	public Vector3d subtract(double scalar)
	{
		Vector3d v = new Vector3d();
		for (int i = 0; i < DIMENSION; i++) {
			v.vec[i] = vec[i] - scalar;
		}

		return v;
	}

	public Vector3d multiply(double scalar)
	{
		Vector3d v = new Vector3d();
		for (int i = 0; i < DIMENSION; i++) {
			v.vec[i] = vec[i] * scalar;
		}

		return v;
	}

	public Vector3d divide(double scalar)
	{
		if (scalar == 0.0) {
			return null;
		}

		Vector3d v = new Vector3d();
		for (int i = 0; i < DIMENSION; i++) {
			v.vec[i] = vec[i] / scalar;
		}

		return v;
	}

	public void addEqual(double scalar)
	{
		for (int i = 0; i < DIMENSION; i++) {
			vec[i] += scalar;
		}
	}

	public void subtractEqual(double scalar)
	{
		for (int i = 0; i < DIMENSION; i++) {
			vec[i] -= scalar;
		}
	}

	public void multiplyEqual(double scalar)
	{
		for (int i = 0; i < DIMENSION; i++) {
			vec[i] *= scalar;
		}
	}

	public void divideEqual(double scalar)
	{
		if (scalar == 0.0) {
			return;
		}

		for (int i = 0; i < DIMENSION; i++) {
			vec[i] /= scalar;
		}

	}

	public Vector3d add(Vector3d v)
	{
		Vector3d dstV = new Vector3d();
		for (int i = 0; i < DIMENSION; i++) {
			dstV.vec[i] = vec[i] + v.vec[i];
		}

		return dstV;
	}

	public Vector3d subtract(Vector3d v)
	{
		Vector3d dstV = new Vector3d();
		for (int i = 0; i < DIMENSION; i++) {
			dstV.vec[i] = vec[i] - v.vec[i];
		}

		return dstV;
	}

	public void addEqual(Vector3d v)
	{
		for (int i = 0; i < DIMENSION; i++) {
			vec[i] += v.vec[i];
		}
	}

	public void subtractEqual(Vector3d v)
	{
		for (int i = 0; i < DIMENSION; i++) {
			vec[i] -= v.vec[i];
		}
	}

	public boolean equals(Object o)
	{
		if (!this.getClass().equals(o.getClass())) {
			return false;
		}

		Vector3d v = (Vector3d)o;

		for (int i = 0; i < DIMENSION; i++) {
			if (vec[i] != v.vec[i]) {
				return false;
			}
		}

		return true;
	}

	public Vector3d copy()
	{
		return (new Vector3d(vec));
	}

	public Object clone()
	{
		return (new Vector3d(vec));
	}

	public double dot(Vector3d v)
	{
		double dot = 0.0;
		for (int i = 0; i < DIMENSION; i++) {
			dot += vec[i] * v.vec[i];
		}

		return dot;
	}

	public Vector3d cross(Vector3d v)
	{
		Vector3d cross = new Vector3d();
		for (int i = 0; i < DIMENSION; i++) {
			cross.vec[i] =
				vec[(i + 1) % DIMENSION] * v.vec[(i + 2) % DIMENSION] - 
				vec[(i + 2) % DIMENSION] * v.vec[(i + 1) % DIMENSION];
		}

		return cross;
	}

	public double length()
	{
		return Math.sqrt(dot(this));
	}

	public void nigate()
	{
		for (int i = 0; i < DIMENSION; i++) {
			vec[i] = - vec[i];
		}
	}

	public double normalize()
	{
		double l = length();
		if (l == 0.0) {
			return 0.0;
		}

		for (int i = 0; i < DIMENSION; i++)	{
			vec[i] /= l;
		}

		return l;
	}

	public void print()
	{
		String message =
			"[x, y, z] = " + "[" + vec[0] + ", " + vec[1] + ", " + vec[2] + "]";

		System.out.println(message);
	}

	public String toString()
	{
		String message =
			"[x, y, z] = " + "[" + vec[0] + ", " + vec[1] + ", " + vec[2] + "]";

		return message;
	}

	public double angle(Vector3d v)
	{
		Vector3d normal1 = new Vector3d(vec);
		normal1.normalize();
		Vector3d normal2 = new Vector3d(v.vec);
		normal2.normalize();

		double cos = normal1.dot(normal2);
		double sin = normal1.cross(normal2).length();

		return (radianToDegree(Math.atan2(sin, cos)));
	}

	// PROTECTED METHODS
	protected double degreeToRadian(double degree)
	{
		return (degree * Math.PI / 180.0);
	}

	protected double radianToDegree(double radian)
	{
		return (radian * 180.0 / Math.PI);
	}

	protected void printErrorMessage(String methodName)
	{
		String errorMessage = "ERROR : ";
		errorMessage += ("IN CLASS [" + getClass().getName() + ", ");
		errorMessage += ("IN METHOD [" + methodName + "]");

		System.err.println(errorMessage);
	}
}

