//------------------------------------------------------------//
//                                                            //
//  2xNgNX                                       //
//                                                            //
//	Hiromu Itoh     2005                                      //
//                                                            //
//------------------------------------------------------------//
public class Vector2d implements Cloneable
{
	// PUBLIC MEMBERS
	public final static int DIMENSION = 2;

	// PRIVATE MEMBERS
	private double vec[];

	// CONSTRUCTORS
	public Vector2d()
	{
		this(0.0, 0.0);
	}

	public Vector2d(double x, double y)
	{
		vec = new double[DIMENSION];

		vec[0] = x;
		vec[1] = y;
	}

	public Vector2d(double v[])
	{
		vec = new double[DIMENSION];

		vec[0] = v[0];
		vec[1] = v[1];
	}

	// PUBLIC METHODS
	public Vector2d rotate(double angle)
	{
		Vector2d v = new Vector2d();

		double radian = degreeToRadian(angle);
		double cos = Math.cos(radian);
		double sin = Math.sin(radian);

		v.vec[0] = cos * vec[0] + - sin * vec[1];
		v.vec[1] = sin * vec[0] + cos * vec[1];

		return v;
	}

	public void rotateEqual(double angle)
	{
		double v[] = new double[DIMENSION];

		double radian = degreeToRadian(angle);
		double cos = Math.cos(radian);
		double sin = Math.sin(radian);

		v[0] = cos * vec[0] + - sin * vec[1];
		v[1] = sin * vec[0] + cos * vec[1];

		setValue(v);
	}

	public Vector2d translate(Vector2d trans)
	{
		return add(trans);
	}

	public void translateEqual(Vector2d trans)
	{
		addEqual(trans);
	}

	public Vector2d scale(double scale)
	{
		return multiply(scale);
	}

	public void scaleEqual(double scale)
	{
		multiplyEqual(scale);
	}

	public double setLength(double length)
	{
		double l = normalize();
		multiplyEqual(length);

		return l;
	}

	public void set(int index, double value)
	{
		if (index < 0 || index >= DIMENSION) {
			return;
		}

		vec[index] = value;
	}

	public double get(int index)
	{
		if (index < 0 || index >= DIMENSION) {
			return 0.0;
		}

		return vec[index];
	}

	public void setX(double value)
	{
		set(0, value);
	}

	public void setY(double value)
	{
		set(1, value);
	}

	public double getX()
	{
		return get(0);
	}

	public double getY()
	{
		return get(1);
	}

	public void setValue(double x, double y)
	{
		setX(x);
		setY(y);
	}

	public void setValue(double v[])
	{
		for (int i = 0; i < DIMENSION; i++) {
			set(i, v[i]);
		}
	}

	public void getValue(double v[])
	{
		for (int i = 0; i < DIMENSION; i++) {
			v[i] = get(i);
		}
	}

	public Vector2d minus()
	{
		Vector2d v = new Vector2d();
		for (int i = 0; i < DIMENSION; i++) {
			v.vec[i] = - vec[i];
		}

		return v;
	}

	public Vector2d add(double scalar)
	{
		Vector2d v = new Vector2d();
		for (int i = 0; i < DIMENSION; i++) {
			v.vec[i] = vec[i] + scalar;
		}

		return v;
	}

	public Vector2d subtract(double scalar)
	{
		Vector2d v = new Vector2d();
		for (int i = 0; i < DIMENSION; i++) {
			v.vec[i] = vec[i] - scalar;
		}

		return v;
	}

	public Vector2d multiply(double scalar)
	{
		Vector2d v = new Vector2d();
		for (int i = 0; i < DIMENSION; i++) {
			v.vec[i] = vec[i] * scalar;
		}

		return v;
	}

	public Vector2d divide(double scalar)
	{
		if (scalar == 0.0) {
			return null;
		}

		Vector2d v = new Vector2d();
		for (int i = 0; i < DIMENSION; i++) {
			v.vec[i] = vec[i] / scalar;
		}

		return v;
	}

	public void addEqual(double scalar)
	{
		for (int i = 0; i < DIMENSION; i++) {
			vec[i] += scalar;
		}
	}

	public void subtractEqual(double scalar)
	{
		for (int i = 0; i < DIMENSION; i++) {
			vec[i] -= scalar;
		}
	}

	public void multiplyEqual(double scalar)
	{
		for (int i = 0; i < DIMENSION; i++) {
			vec[i] *= scalar;
		}
	}

	public void divideEqual(double scalar)
	{
		if (scalar == 0.0) {
			return;
		}

		for (int i = 0; i < DIMENSION; i++) {
			vec[i] /= scalar;
		}

	}

	public Vector2d add(Vector2d v)
	{
		Vector2d dstV = new Vector2d();
		for (int i = 0; i < DIMENSION; i++) {
			dstV.vec[i] = vec[i] + v.vec[i];
		}

		return dstV;
	}

	public Vector2d subtract(Vector2d v)
	{
		Vector2d dstV = new Vector2d();
		for (int i = 0; i < DIMENSION; i++) {
			dstV.vec[i] = vec[i] - v.vec[i];
		}

		return dstV;
	}

	public void addEqual(Vector2d v)
	{
		for (int i = 0; i < DIMENSION; i++) {
			vec[i] += v.vec[i];
		}
	}

	public void subtractEqual(Vector2d v)
	{
		for (int i = 0; i < DIMENSION; i++) {
			vec[i] -= v.vec[i];
		}
	}

	public boolean equals(Object o)
	{
		if (!this.getClass().equals(o.getClass())) {
			return false;
		}

		Vector2d v = (Vector2d)o;

		for (int i = 0; i < DIMENSION; i++) {
			if (vec[i] != v.vec[i]) {
				return false;
			}
		}

		return true;
	}

	public Vector2d copy()
	{
		return (new Vector2d(vec));
	}

	public Object clone()
	{
		return (new Vector2d(vec));
	}

	public double dot(Vector2d v)
	{
		double dot = 0.0;
		for (int i = 0; i < DIMENSION; i++) {
			dot += vec[i] * v.vec[i];
		}

		return dot;
	}

	public double length()
	{
		return Math.sqrt(dot(this));
	}

	public void nigate()
	{
		for (int i = 0; i < DIMENSION; i++) {
			vec[i] = - vec[i];
		}
	}

	public double normalize()
	{
		double l = length();
		if (l == 0.0) {
			return 0.0;
		}

		for (int i = 0; i < DIMENSION; i++)	{
			vec[i] /= l;
		}

		return l;
	}

	public void print()
	{
		String message =
			"[x, y] = " + "[" + vec[0] + ", " + vec[1] + "]";

		System.out.println(message);
	}

	public String toString()
	{
		String message =
			"[x, y] = " + "[" + vec[0] + ", " + vec[1] + "]";

		return message;
	}

	public double angle(Vector2d v)
	{
		Vector2d normal1 = new Vector2d(vec);
		normal1.normalize();
		Vector2d normal2 = new Vector2d(v.vec);
		normal2.normalize();

		double cos = normal1.dot(normal2);

		return (radianToDegree(Math.acos(cos)));
	}

	// PROTECTED METHODS
	protected double degreeToRadian(double degree)
	{
		return (degree * Math.PI / 180.0);
	}

	protected double radianToDegree(double radian)
	{
		return (radian * 180.0 / Math.PI);
	}

	protected void printErrorMessage(String methodName)
	{
		String errorMessage = "ERROR : ";
		errorMessage += ("IN CLASS [" + getClass().getName() + ", ");
		errorMessage += ("IN METHOD [" + methodName + "]");

		System.err.println(errorMessage);
	}
}


